/*********************************************************************
 *
 *                DCC Mobile Decoder Version 1.0
 *
 *********************************************************************
 * FileName:        main.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC16F628A
 * Compiler:        XC8
 * Company:         My personal use
 *
 * Notes:						
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Ernesto Pareja       23/12/07    Original.
 * Ernesto Pareja				28/08/09 		Corrected the Function group bits positions
 ********************************************************************/

#include <xc.h>
#include "IOConfig.h"


// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

// CONFIG
#pragma config FOSC = HS        // Oscillator Selection bits (HS oscillator: High-speed crystal/resonator on RA6/OSC2/CLKOUT and RA7/OSC1/CLKIN)
#pragma config WDTE = ON        // Watchdog Timer Enable bit (WDT enabled)
#pragma config PWRTE = ON       // Power-up Timer Enable bit (PWRT enabled)
#pragma config MCLRE = OFF      // RA5/MCLR/VPP Pin Function Select bit (RA5/MCLR/VPP pin function is digital input, MCLR internally tied to VDD)
#pragma config BOREN = OFF      // Brown-out Detect Enable bit (BOD disabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable bit (RB4/PGM pin has digital I/O function, HV on MCLR must be used for programming)
#pragma config CPD = OFF        // Data EE Memory Code Protection bit (Data memory code protection off)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit (Code protection off)


//__CONFIG(HS & WDTEN  & MCLRDIS & PWRTEN & BORDIS & UNPROTECT & LVPDIS);

/*****  M A C R O   D E F I N I T I O N S ********************/
#define HIGH(x) 	((unsigned)(x) >> 8)
#define LOW(x)  	((x) & 0xFF)
#define BTF(x,b)	((x) ^= (1 << (b)))	// Bit Toggle
#define BCF(x,b)	((x) &= ~(1 << (b)))	// Bit Clear
#define BSF(x,b)	((x) |= (1 << (b)))	// Bit Set
#define BTSTF(x,b)	(!!((x) & (1 << (b))))  // Bit test

/*********** E E P R O M   F A C T O R Y   P R E L O A D*************/
					//	Dummy CV1		CV2(Vst)	CV3(accel)-CV4(decel)	CV5(VMax)	CV6(Vmid)   CV7(version)	
__EEPROM_DATA(0x55, 0x03, 0x00,     0x07, 		  0x07, 		0xFF, 		0x00, 			0x03);

// Version 2 : Working version
// Version 3 : Corrected the functions group bits
					
					   //	CV8		CV11  CV19	CV29		CVx		CVx		CVx		CVx		CVx 	CVx
__EEPROM_DATA(0xA5, 0xFF, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00);
__EEPROM_DATA(0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 );

/*********** C O N S T A N T S *********************/
#define FOSC								19 // 8 //20 // 20MHz
#define PRESFOSC						19.660 

#if FOSC == 20
	#define	TMR0POSTSCALER			4
	#define TMR1PRESCALER				4 // For DCC Timming 255 - ((110 * (4/Fosc)) / TMR0POSTSCALER)
	#define PWM_PERIOD_US				110
	#define PWM_CLICK_TIMER			(unsigned char)(255-(0.5+((PWM_PERIOD_US*PRESFOSC)/(4*TMR0POSTSCALER)))) + 11 // The 11 are the instructions until it clears the timer in the interrupt routine
	#define	PWM_MAX							254
	#define	ACCEL_DECEL_BASE		19 // Value for Timer2 to count 1ms @ 20Mhz
	
	// DCC TIMMING CONSTANTS
	#define	_1UPPER_LIMIT				88//35 //88	// 70 us
	#define	_1LOWER_LIMIT				50//20 //50	// 40 us
	#define	_0LOWER_LIMIT				115//46 //115	// 93 us
	#define	_0UPPER_LIMIT				250//100//250 // 200 us
	
//	#define	_28SPEED_ACC				255	// Time constant for TMR2 to interrupt every 13 ms
//	#define	_128SPEED_ACC				137 // Time constant for TMR2 to interrupt every 7 ms
#elif FOSC == 19
	#define	TMR0POSTSCALER			4
	#define TMR1PRESCALER				4 // For DCC Timming
	#define PWM_PERIOD_US				110
	#define PWM_CLICK_TIMER 		(unsigned char)(255-(0.5+((PWM_PERIOD_US*PRESFOSC)/(4*TMR0POSTSCALER)))) + 11 // The 11 are the instructions until it clears the timer in the interrupt routine

	#define	PWM_MAX							254
	#define	ACCEL_DECEL_BASE		19 // Value for Timer2 to count 1ms @ 20Mhz
	
	// DCC TIMMING CONSTANTS
	#define	_1UPPER_LIMIT				(unsigned char)(0.5+(70*PRESFOSC)/(4*TMR0POSTSCALER))  //86 dec  // 70 us
	#define	_1LOWER_LIMIT				(unsigned char)(0.5+(40*PRESFOSC)/(4*TMR0POSTSCALER))  //49 dec  // 40 us
	#define	_0LOWER_LIMIT				(unsigned char)(0.5+(93*PRESFOSC)/(4*TMR0POSTSCALER))  //114 dec // 93 us
	#define	_0UPPER_LIMIT				(unsigned char)(0.5+(200*PRESFOSC)/(4*TMR0POSTSCALER))  //246 // 200 us
	
//	#define	_28SPEED_ACC				255	// Time constant for TMR2 to interrupt every 13 ms
//	#define	_128SPEED_ACC				137 // Time constant for TMR2 to interrupt every 7 ms
#endif

#define RISING_EDGE					1
#define FALLING_EDGE				0
#define _128SPEED						1			
#define	_28SPEED						0


// DCC STATE MACHINE CONSTANTS
#define PREAMBLE_STATE			0
#define WAIT_INIT_BIT				1  // Init packet bit
#define	FIRST_BYTE_STATE		2
#define	WAIT_START1_BIT			3
#define	SECOND_BYTE_STATE		4
#define	WAIT_START2_BIT			5
#define	THIRD_BYTE_STATE 		6
#define	WAIT_START3_BIT			7
#define	FORTH_BYTE_STATE 		8
#define	WAIT_START4_BIT			9
#define	FIFTH_BYTE_STATE 		10
#define	WAIT_START5_BIT			11

/****** DCC PACKET COMMANDS (SEE 9.2.1 NMRA Standard) ***************/
#define	DECODER_CTRL_CMD		0b00000000	// 0000XXXX
#define	CONSIST_CTRL_CMD		0b00010000	// 0001XXXX		
#define ADV_INSTRUCT_CMD		0b00100000	// 001XXXXX
#define SPEED_128_CMD				0b00111111	// 00111111  Packet for 128 Steps Configuration
#define	SPEED_DIR_CMD				0b01000000	// 01XXXXXX   (010=Reverese   011=Forward)
#define	SPEED_DIR_CMD2			0b01100000	// 01XXXXXX   (010=Reverese   011=Forward)
#define FUNC_GRP1_CMD				0b10000000	// 100XXXXX
#define FUNC_GRP2_CMD				0b10100000	// 101XXXXX
#define	FUNC_GRP2_A_CMD			0b10110000	// 1011XXXX	  (F5-F8)
#define	FUNC_GRP2_B_CMD			0b10100000	// 1010XXXX		(F9-F12)
#define	FEATURE_EXP_CMD			0b11000000	// 110XXXXX
#define	CV_ACCESS_CMD				0b11100000	// 111XXXXX	 (Represented in the next to commands
#define	CV_WRITE_CMD				0b11101100	// 11101100  (CV Access - WriteByte)
#define	CV_READ_CMD					0b11100100	// 11100100  (CV Access - VerifyByte)
#define	CV_BIT_WRITE_CMD		0b11101000	// 11100100  (CV Access - Bit Manipulation)
#define	CV_WRITE_DMODE_CMD	0b01111100	// 011111XX  (CV Access Direct Mode - WriteByte)
#define	CV_READ_DMODE_CMD		0b01110100	// 011101XX  (CV Access Direct Mode - ReadByte)
#define	CV_WRITE_PHYS_REG_CMD	0b01111000	// 01111RRR  (CV Physical Register instruction- WriteByte)
#define	CV_WRITE_ADDR_ONLY_MODE_CMD	0b01111000	// 01111XXX  (CV Access Address Only Mode- WriteByte)
#define	CV_READ_ADDR_ONLY_MODE_CMD	0b01110000	// 011100XX  (CV Access Address Only Mode - WriteByte)
#define RESET_CMD						0b00000000	// 00000000  Reset command (Must stop inmediatly
#define SERVICE_LOCK_CMD		0b11111001  // Service lock command 0xFA (See RP 9.2.3 Apendix B)

/*********** G E N E R A L   O P E R A T I O N    C O N S T A N T S **********/
#define MAX_ACK_TIME				20// Mesured in ms is the MAX time NMRA standar says the motor should be turned on for ACK (NMRA RP 9.2.3 part D)
#define INIT_EEPROM_ADDR		1 // The first EEPROM Address where CV1 will be placed and read from
#define MAX_CV_VARIABLES		11 // The number of CVs stored in EEPROM

/*********** F U N C T I O N S    P R O T O T I P E S**********/
void	AnalizeDCC(void);
void CheckLights(void);
unsigned char DecodeSpeed(char CodedSpeed);
void eeprom_write2(char Addr, char Data) ;
void CheckDCTimeOut(void);


/*********** D A T A   S T R U C T U R E S*********************/
typedef union {
	unsigned char _byte;
	struct {
		unsigned	DCC_bit_state :1;
		unsigned	CheckHalfBit  :1;
		unsigned	FirstHalfBit	:1;
		unsigned	BitReceived 	:1; // stores the second half bit and Stores the bit received from DCC according to timming 
		unsigned	CheckFullBit	:1; // Indicates a complete bit has been received OK (Timing is ok)
		unsigned	Ack						:1; // Indicates it has to accknowledge the command
		unsigned	MustStop			:1; // Emergency stop
		unsigned	DCMode				:1; // Indicates the loco is operating in DC mode
		unsigned	DCCServiceMode:1;	// Idicates a ServiceMode reset has been received
		unsigned	RecalcPWM			:1; // Must calculate the PWM Output timming in Main
		unsigned	MayProcServMode:1; // Flag to allow AddressOnly Mode andPhysical Register Addressing Service Mode instructions
		unsigned	CheckLights		:1;	// Must check Lights in Main. Also used to check the Accel Decel timming
		unsigned	MustProcessCmd:1; // Indicates that a valid DCC packet has been anailzed and allow the main routine to recaluculate PWM out and other time consuming tasks

	} flag;	
} TDCCFlags;
	
/****** Configuration of CV29 **************/
typedef union{
	char	_byte;
	struct {
		unsigned	LocoDirection		:	1;
		unsigned	FL_location			: 1;
		unsigned	PowerConvertion	: 1;
		unsigned	Bi_comunication	:	1;
		unsigned	SpeedTable			: 1;
		unsigned	ExtAddressing		: 1;
		unsigned	Reserved				: 1;
		unsigned	TypeOfDecoder		: 1; // 0=Multifunctional  1=Accesory Decoder (See CV#541 for description for bits 0-6
	} flags;
} TCV29;	

typedef union {
		unsigned char _byte;
		struct {
			unsigned F1		:1;
			unsigned F2		:1;
			unsigned F3		:1;
			unsigned F4		:1;
			unsigned FL		:1; // Lights control F0
		}Fun1;

		struct { // Contains function2 groups 1(F5-F8) and 2 (F9-F12)
			unsigned F5		:1;
			unsigned F6		:1;
			unsigned F7		:1;
			unsigned F8		:1;
			unsigned F9		:1;
			unsigned F10	:1;
			unsigned F11	:1; 
			unsigned F12	:1;
		}Fun2;	
} TFuncGroup;


typedef union {
	char	_byte;
	struct {
		unsigned	DirBit 			: 1;
		unsigned	RealDirBit	: 1;  // To allow deceleration and changing dir
		unsigned	SpeedMode 	:	1;
	} bits;
}TLocoFlags;

typedef struct {
	char Address; // CV1
	char CV2;		// VStart
	char CV3;		// Acceleration Rate
	char CV4;		// Deceleration Rate
	char CV5;		// VHigh
	char CV6;		// VMid (Apply only on 128 Speed Step)
	char CV7;		// Manufacturer Version
	char CV8;		// Manufactured ID
	char CV11;	// Packet Time out Value to stop when not receiving a DCC packet address to the decoder
	char CV19;	// Consist Address
	TCV29 CV29;	// Configuration Variable
	char Speed;
	TLocoFlags Flags;
	TFuncGroup	Func1;
	TFuncGroup	Func2;
}	TLocoData;



//static volatile unsigned int TMR1	@ 0x0E;
char PWM_Timer;
char PWM_TOn;
char PWM_Period;
char PWM_RealTOn;  // To control acceleration in PWM units not in DCC speed units
TLocoData LocoData;
TDCCFlags	DCCFlags;


/****** DCC Variables ************/
char buffer[5]; // Contains the bytes received from DCC (Max number of bytes)
char Lastbuffer[5]; // To copy the last complete DCC packet received to check if was the same
char DCC_Byte;  // Where the program stores the bits received
char DCC_State;
char CVNumber;
char CVValue;	// Stores the CV Value received in a CV Write Command
char CVWriteInst;	// Indicates how many CV Write Packets received simultaneusly
char PagePresetCount; // Checks how many Page Preset Packets has been received for Service Mode Instructions
char ServModeCount;	// Counts how many Service Mode Packets has been received
char ServModeTimer;	// To detect 20ms of not receiving any packet and exits ServiceMode (Uses TMR2 1ms interrupt)
char BytesReceived;
char BitsReceived; // To check how many bits has been received in a normal byte
char PreambleBits; // Counter to detect the preamble bits
char HalfBitTime; // To save TMR1L value that represents the time taken by the half bit to change state
char AccelTime; // To count on TMR2 interrupts for accel configuration of CV3
char DCTimeOut;
char ACKTime; // Counts in a 1ms base for maximum time to apply voltaje to the motor to acknowledge

char EEpromCheck1;
char EEpromCheck2;


	 
/**************************************************************
  Vector de atencin de interrupciones.
***************************************************************/
void interrupt int_vct(void) {
	char BitEdgeStatus;
	if (TMR1IF == 1) {
		TMR1IF = 0;	
		DCCFlags.flag.DCC_bit_state=0;
		RA2 ^= 1;
		OPTION_REG	= 0b00000001;  // Preescaler x 2 assigned to TMR0
		TMR0 = PWM_CLICK_TIMER;
		PEIE = 1; // To enable Periferial Interrupts (CCP1)
		T0IE = 1; // Timer 0 Interrupt Enabled
		TMR1IE = 1; // Enable the Timer 1 interrupt to detect long idle bits and restart DCC bit sequence
		TMR2IE = 1; // Enable the Timer 2 interrupt for acc & decceleration timming
		CCP1IE = 1; // Capture Interrupt Enabled - for dcc signal timming
		CMCON = 0x07; // Disable the compare module and enable portA as digital
		if (DCC_Input == 1)
		  CCP1CON = 0b00000100; // Capture on Falling Edge (for now
		else
		  CCP1CON = 0b00000101; // Capture on Rising Edge (for now
		
		T1CON	= 0b00100101; // Preescaler x 4
		TMR1 = 0; // Clears the Timer 1 (used to capture the DCC timming sequence)
		PR2	 = ACCEL_DECEL_BASE; // 1 ms timer 
		T2CON = 0b01111111;
		TMR2 = 0;
		
	}
	
	if (CCP1IF==1) {  // Capture time for DCC Timming
		CCP1IF =0;
		TMR1 = 0;
		if (DCC_Input == 1) { // PORTB,3
			BitEdgeStatus =0;	// FALLING EDGE
		} else {
			BitEdgeStatus =1;	// RISING_EDGE
		}
	  CCP1CON = 0;
	  CCP1CON = 0b00000100 + BitEdgeStatus; // Changes the Edge of the Capture
		
		HalfBitTime = CCPR1L;
		
		DCCFlags.flag.CheckHalfBit=1; // Leaves a message to Main routine to check the bit time to determine if is 0 or 1
	} // End of CCP1 Interrupt (DCC signal change detection)
	
	if (T0IF==1) {  // PWM Timer Interrupt Service
   	T0IF =0;
   	TMR0  = PWM_CLICK_TIMER;
   	PWM_Timer++;
   	if (PWM_RealTOn != 0) { // Is real speed diferent from zero?
	  	if (PWM_Timer >= PWM_Period) { 
		  	PWM_Timer=0;
				if	(LocoData.Flags.bits.RealDirBit==1){
		  		PWM_Port2 = 0;
		  		PWM_Port1 = 1;	
		  	} else {
		  		PWM_Port1 = 0;	
		  		PWM_Port2 = 1;	
		  	}
		  }
	   	if (PWM_Timer > PWM_RealTOn){ // Duty Cycle is over? (if yes, clears the PWM Output)
				PWM_Port1=0;
				PWM_Port2=0;
		  }
		}
	} // End of  TMR0 Interrupt (PWM)

	// ****** 	interrupt every 1ms to check Aceleration and Desceleration 
	// ******   base and DCC Time out to switch to DC mode and check 
	// ******   service mode time out. It is checked in main routine
	if (TMR2IF) {
		TMR2IF =0;
		AccelTime++;
		DCCFlags.flag.CheckLights=1;

	} // End of Timer 2 Interrupt (Acceleration and Deceleration)
	
}


/******************************************************************************
 * Function:        void RestoreEEPromValues(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        Restores the Factory State to all available CVs
 *	 CV1		CV2		CV3		CV4		CV5		CV7		CV8		CV11
 *  (0x03, 0x00, 0x07, 0x07, 0x7F, 0x7F, 0xA5, 0xFF);
 *   CV19	 CV29		CVx		CVx		CVx		CVx		CVx 	CVx
 *  (0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);
 * Note:            None
 *****************************************************************************/
void RestoreEEPromValues(void) {
	const char InitValues[10] = {3,0,7,7,PWM_MAX,0 ,0x01, 0xA5,0,2};
	char i;
	char * PLocoData;
	
	PLocoData = &LocoData.Address;
	for (i=0; i<MAX_CV_VARIABLES ; i++) {
		EEpromCheck1 = 0x5A;
		EEpromCheck2 = 0xA5;
		eeprom_write2(i+INIT_EEPROM_ADDR,InitValues[i]);	
//		eeprom_write2(i,InitValues[i]);	
		*PLocoData = InitValues[i];
		PLocoData++;
	}
}	



/******************************************************************************
 * Function:        void SaveLastBuffer(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        Saves the buffer values to check identical packets later
 *
 * Note:            None
 *****************************************************************************/
void SaveLastBuffer(void) {
	Lastbuffer[0] = buffer[0]; // Saves the actual data
	Lastbuffer[1] = buffer[1];
	Lastbuffer[2] = buffer[2];
	Lastbuffer[3] = buffer[3];
	Lastbuffer[4] = buffer[4];
}	


/******************************************************************************
 * Function:        char IsIdenticalPacket(void)
 * PreCondition:    None
 * Input:           None
 * Output:          0xFF = Identical Packets
 *									0x00 = Different Packets
 * Side Effects:    None
 * Overview:        Checks for two identical consecutive DCC packets 
 *
 * Note:            None
 *****************************************************************************/
char IsIdenticalPacket(void){
  if (buffer[0] == Lastbuffer[0] && buffer[1]==Lastbuffer[1] &&
			buffer[2] == Lastbuffer[2] && buffer[3]==Lastbuffer[3]) 
		return 0xFF;	// Identical Packets
	else
	  return 0x00;	// Not identical
}	



/******************************************************************************
 * Function:        void AssignCVValue(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        This functions decode a  CVAddress and stores it
 *									in the right EEProm Address
 * Note:            None
 *****************************************************************************/
void AssignCVValue(char CVAddr, char CV){
	DCCFlags.flag.Ack = 1; // Loco has to acknowledge the command
	// Setting again the CVNumber in each case statement is to 
	// know the EEProm Address that the CV is stored on. 
	// 255 indicates that CV must not be written in EEProm
	// When a case doesnt contain a CVNumber reassigment is because it would be the same
	switch (CVAddr) {
		case 0:  LocoData.Address 		= CV; break; // Loco Address
		case 1:  LocoData.CV2					= CV; break; // VStart
		case 2:  LocoData.CV3					= CV; break;	// Accel
		case 3:  LocoData.CV4					= CV; break;	// Decel
		case 4:  LocoData.CV5					= CV; break;	// VHigh
		case 5:  LocoData.CV6					= CV; break;	// VMid

		//case 6:  LocoData.CV7				= CV; break; // Version #
		case 7: { // If attempt to write 8 to CV8 the is a reset Factory Default Values
			CVAddr=255; // Indicates that must not be written in EEProm
			if (CV == 8)
				RestoreEEPromValues();
				DCCFlags.flag.Ack = 1;
			break;
		}
		case 10: LocoData.CV11				= CV;CVAddr=8; break; // Packet Time Out value
		case 18: LocoData.CV19				= CV;CVAddr=9; break; // Consist Address
		case 28: LocoData.CV29._byte	= CV;CVAddr=10; break; // Config Address
		default : { CVAddr = 0xFF; DCCFlags.flag.Ack =0; }// Doesnt acknowledge because no implemented CV was received
	} // End del switch (CVNumber+1)
	if (CVAddr < 10) {
		EEpromCheck1 = 0x5A; // Sets the eemprom write control to prevent undesired operation
		EEpromCheck2 = 0xA5;
		eeprom_write2(CVAddr+INIT_EEPROM_ADDR,CV); // See htc.h for prototype
	}
}


/******************************************************************************
 * Function:        void eeprom_write2(char Addr, char Data)
 * PreCondition:    None
 * Input:           Addr : is the EEPROM Address you want to write to
 *									Data : Self explanatory
 * Output:          None
 * Side Effects:    None
 * Overview:        This function prepare some checking to be sure there is no
 *									program malfunction and calls the EEPROM_WRITE MACRO. 
 *									Defined in pic.h
 *									in the right EEProm Address
 * Note:            None
 *****************************************************************************/
void eeprom_write2(char Addr, char Data) {
		
	if ( EEpromCheck1 == 0x5A &&	EEpromCheck2 == 0xA5) {
		GIE = 0; // Dissable all the interrupts during a EEPROM WRITE 
		EEPROM_WRITE(Addr,Data);  	
		WREN = 0; // Clears the WREN bit to prevent undesired EEPROM programming
		GIE = 1;
		EEpromCheck1 = 0;
		EEpromCheck2 = 0;
	} else {
		asm( "goto 0x0000");
	}
}	
	
/******************************************************************************
 * Function:        void	AnalizeDCC(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        Decode a received DCC Packet
 *
 * Note:            None
 *****************************************************************************/
void	AnalizeDCC(void){
	char TempCommand; // To help decoding DCC commands
	char PhysicalRegister; // To decode Physical Register Service Mode Command
	volatile char CheckSum;
	volatile char i;
	
	CheckSum =0;
	for (i=0;i<BytesReceived-1;i++) {
		CheckSum ^= buffer[i];
	}
	if (CheckSum == buffer[i]) { // If checksum is OK Then continue
		if (PreambleBits <= 20 ) { // Short Preamble
			if ((buffer[0] == 0) || ((buffer[0] == LocoData.Address && LocoData.CV19 == 0)|| 
					(buffer[0] == LocoData.CV19))) { // Broadcast, short address or consist address
				TempCommand = buffer[1] & 0b11100000;
				if (buffer[1] != CV_WRITE_CMD) {
					CVWriteInst=0; //Clear the CV instructions if not CV_WRITE_CMD
				}
				if (buffer[1] != RESET_CMD){
					DCCFlags.flag.DCCServiceMode = 0;// Not a Service mode packet
					DCCFlags.flag.MayProcServMode = 0;
					PagePresetCount=0;
					ServModeCount=0;
				}
				switch (TempCommand) {
					case DECODER_CTRL_CMD:{
						switch (buffer[1]) {
							case 0b00000000: case 0b00000001:{ // Reset Packet & Hard Reset
								DCCFlags.flag.MustStop = 1; 
								LocoData.Speed = 0; // inmediate stop
								LocoData.Flags.bits.SpeedMode = _28SPEED;
								LocoData.Flags.bits.RealDirBit = 0;
								LocoData.Flags.bits.DirBit =0;
								DCCFlags.flag.DCCServiceMode = 1; // Enters service mode
								ServModeTimer=0; // Resets the Timer for DCC ServiceMode Exit
								
								if (buffer[1] == 0b00000001) { // Hard Reset Packet (CV19=0 and CV29 = 6)
//									eeprom_write2(&LocoData.CV19-&LocoData.Address,0);
//									eeprom_write2(&LocoData.CV29._byte-&LocoData.Address,6);
								}
							}
							case 0b00001111: { // Ack (Motor On for 5 ms)
								DCCFlags.flag.Ack = 1;
								break;
							}
						}
						break;	
					}
					case ADV_INSTRUCT_CMD:{ // 001CCCCC 
						if (buffer[1] == 0b00111111) { // 128 SPEED STEP INSTRUCTION
							LocoData.Flags.bits.SpeedMode = _128SPEED;
							LocoData.Speed = DecodeSpeed(buffer[2] & 0x7F);
						}
						
						break;
					}
					case SPEED_DIR_CMD: case SPEED_DIR_CMD2 :{ // 01DsSSSS 
						LocoData.Flags.bits.SpeedMode = _28SPEED;
						LocoData.Speed = DecodeSpeed(buffer[1] & 0b00111111); // Ensures that no more than 32 value comes here
						break;
					}
					case FUNC_GRP1_CMD:{   // 100DDDDD   (FL to F4)
						LocoData.Func1._byte = buffer[1] & 0b00011111;
						break;	
					}
					case FUNC_GRP2_CMD:{  //101SDDDD  
						if (BTSTF(buffer[1],4)) { // F5 to F8
							LocoData.Func2._byte |= 	buffer[1] & 0x0F; // masks the upper nible (F9 to F12)
						} else { // F9 to F12
							LocoData.Func2._byte |= 	buffer[1] & 0xF0; // masks the lower nible (F5 to F8)
						}
						
						break;
					}
					case CV_ACCESS_CMD:{
						CVWriteInst++; 
						if (CVWriteInst > 1) { // At least 2 instructions must be received to accept command
							CVWriteInst =0;
							if (IsIdenticalPacket() != 0) {
								AssignCVValue(buffer[2],buffer[3]);
								DCCFlags.flag.Ack = 1; // Must do Ack
								ACKTime = 0;
							}// End del check of identical packet
						} else {
							SaveLastBuffer();
						}	// end del if (CVWriteInst >=2)
						break;
					} // end del case CV_WRITE_CMD
					default : return; // Unkown command
				} // End del switch(TempCommand)
			} else { // It was a packet for other decoder address so resets packet and finish
				return;
			}
			
		} else { // Long Preamble for service mode packets
			ServModeTimer=0; // Resets the 20 ms Timeout counter
			switch (buffer[0]) {
				case 0b01111101 : {  // Page Preset Command (0x7D)
					// long-pream 0 01111101 0 00000001 0 01111100 1 
					if (buffer[1] == 1) {
					  PagePresetCount++;
					  if (PagePresetCount >= 5 ) 
					    DCCFlags.flag.MayProcServMode = 1;
					}
				  break;
				}
				case CV_WRITE_DMODE_CMD : {   //0b01111100
					// This kind of packet is : long-pream 0 0111CCAA 0 AAAAAAAA 0 DDDDDDDD 0 EEEEEEEE 1 
					// Doesnt require Page Preset Packets
					CVWriteInst++;
					if (CVWriteInst >= 5) {
						CVWriteInst =0;
						AssignCVValue(buffer[1],buffer[2]);
						DCCFlags.flag.Ack = 1; // Must do Ack
						ACKTime = 0;
					}
				 	break;	
				}
				
				case CV_WRITE_PHYS_REG_CMD : { // 0b01111RRR  where RRR is the register selection
					// long-pream 0 0111CRRR 0 DDDDDDDD 0 EEEEEEEE 1 
					if (DCCFlags.flag.MayProcServMode==1) {
						if (IsIdenticalPacket() != 0) {
							ServModeCount++;
							if (ServModeCount >= 5) { // Minimum of Writes to a specific register accodring to NMRA 9.2.3
								ServModeCount =0; // To ensure only one Write to the register
								PhysicalRegister = buffer[0] & 0x07; // Masks the 3 LSbits for RRR
								switch (PhysicalRegister){ // This command only supports this CVs and the CV7 and 8s
									case 0:  LocoData.Address = CVValue & 0x7F; CVNumber = 0; break;
									case 1:  LocoData.CV2					= buffer[1]; CVNumber = 1; break;
									case 2:  LocoData.CV3					= buffer[1]; CVNumber = 2; break;
									case 3:  LocoData.CV4					= buffer[1]; CVNumber = 3; break;
									case 4:  LocoData.CV29._byte	= buffer[1]; CVNumber = 9; break;
								} // End del switch (PhysicalRegister)
								
								if (CVNumber < 10) {
									EEpromCheck1 = 0x5A;
									EEpromCheck2 = 0xA5;
									eeprom_write2(CVNumber+INIT_EEPROM_ADDR,buffer[1]);
									DCCFlags.flag.Ack = 1; // Must do Ack
									ACKTime = 0;
								}
							} // end of if (ServModeCount >= 5)
						} // end of if (IsIdenticalPacket() != 0)
					} else { // haven't completed all the required Page Preset Packets (5 at least)
						ServModeCount=0;
						DCCFlags.flag.DCCServiceMode=0; // Exits serviceMode	
					}
					SaveLastBuffer();
					break;
				}
			}
		}

		
	} // End del if (CheckSum == buffer[i])
	
	DCCFlags.flag.MustProcessCmd = 1; 
}	

/******************************************************************************
 * Function:        void CalcAccelerate(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        This functions executes from main using a flag raised by TMR2 interrupt
 *
 * Note:            
 *****************************************************************************/
void CalcAccelerate(void){
	signed int SpeedDif; // Diference between Desired Speed and Real Speed
	unsigned char AccelDecelValue;
	unsigned char States;


	SpeedDif = PWM_TOn - PWM_RealTOn ;
	
	// To detect if is accelerating or decelerating to change comparison value
	if (SpeedDif !=0) {
		if (LocoData.Flags.bits.DirBit==1) { // Must go Forward
			if (	LocoData.Flags.bits.RealDirBit==1) { // is going forward
				AccelDecelValue = LocoData.CV3; // Accel Time
			} else {	// is going reverse
				AccelDecelValue = LocoData.CV4; // Decel Time
			}
		}	else { // Must go Reverse
			if (LocoData.Flags.bits.RealDirBit==1) { // Is going forward
				AccelDecelValue = LocoData.CV4; // Decel Time
			} else {
				AccelDecelValue = LocoData.CV3; // Accel Time
			}
		}
	}
	
	if (AccelTime > AccelDecelValue) {
		AccelTime = 0; // This variable is incremented on TMR2 int (every 1ms)
		if (LocoData.Flags.bits.DirBit==1) {
			if (LocoData.Flags.bits.RealDirBit==1) {
				if (SpeedDif > 0) { // Loco is accelerating and going forward
					PWM_RealTOn++; // If Loco going forward and Speed setting is Higher than real speed (Accelerate)
				} else if (SpeedDif < 0) { // Loco decelerating
					PWM_RealTOn--; // If Loco going forward and Speed setting is lower than real speed (Decelerate)
				} 
			} else { // If Loco is going backward (RealDirBit=0) but desired dir is forward (dirBit=1)
				if (PWM_RealTOn != 0) {//
				  PWM_RealTOn--; // Decelerate until 0 and then changes real dir to desired DirBit
				}
				else {
				  LocoData.Flags.bits.RealDirBit=1; // Changes the RealDirBit as DirBit
				  PWM_RealTOn = 0;
				} 
			}
		} else { // If DirBit = 0
			if (LocoData.Flags.bits.RealDirBit==0) {
				if (SpeedDif > 0) { 
					PWM_RealTOn++; // If Loco going backward and Speed setting is Higher than real speed (Accelerate)
				} else if (SpeedDif < 0) {
					PWM_RealTOn--; // If Loco going backward and Speed setting is lower than real speed (Decelerate)
				} 
			} else { // If Loco is going Forward (RealDirBit=1) but desired dir is BackWard(dirBit=0)
				if (PWM_RealTOn != 0 ) {
				  PWM_RealTOn--;
				}
				else {
				  LocoData.Flags.bits.RealDirBit=0; // Changes the RealDirBit as DirBit
				  PWM_RealTOn=0;
				}
			}
		} // end of the else part of if (LocoData.Flags.bits.DirBit==1) 			
	} // End del if (AccelTime > LocoData.CV3)
}



/******************************************************************************
 * Function:        void CheckLights (void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        Detects the real direction of the Loco and turn
 *									the lights if the FL function is ON
 *
 * Note:            One improvement is implement CV55 to configure
 *									the lights function bit.
 *****************************************************************************/
void CheckLights (void) {
	if (LocoData.Func1.Fun1.FL == 1 || DCCFlags.flag.DCMode==1  ) {
		if (LocoData.Flags.bits.RealDirBit == 1) {
			LED_FRONT = 1;
			LED_REAR = 0;	
		} else {
			LED_FRONT = 0;
			LED_REAR = 1;	
		}
	} else {
			LED_FRONT = 0;
			LED_REAR = 0;	
	}
	if (LocoData.Func1.Fun1.F1 == 1 ) {
		LED_EXTRA = 1;	
	}	else {
		LED_EXTRA = 0;	
	}
}	


/******************************************************************************
 * Function:        unsigned char CalculatePWMOut(char DCCSpeed)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        The real formulas should be
 * 			28 Speed		PWM = (LocoData.Speed*(LocoData.CV5-LocoData.CV2)/28)  + LocoData.CV2;
 *		 128 Speed		PWM = (LocoData.Speed*(LocoData.CV5-LocoData.CV2)/128) + LocoData.CV2;
 *									Those formulas consume a lot of program steps (560 instructions) so is
 *									not convenient to stop the program calculating PWM for so long
 *									There is an alternate way to calculate the Steps for the PWM Output but only for the 28 speed step
 *									It was taken from the Mark Cesle project http://technology.niagarac.on.ca/people/mcsele/dcc.htm
 *
 *			28 Speed		step is used in the 28 speed step
 *									instruction to determine speed curve.
 *									step = 151 X (CV5-CV2) / 256
 *									PWM_TOn = (### - 1)  X (CV5-CV2) / 27 + CV2
 *									        = (### - 1)  X step / 16 + CV2    Replacing equations you have a very aproximate value (Not exact)
 *									The (### -1) is to allow
 * Note:            
 *****************************************************************************/
unsigned char CalculatePWMOut(void) {
	int PWM;
	int temp;
	char result;

	result = 0; // Assume 0 as a result

	if (LocoData.Flags.bits.SpeedMode == _28SPEED) {
		if (LocoData.Speed > 3) {
			temp = 151*(LocoData.CV5-LocoData.CV2);
			PWM = (LocoData.Speed-4) * HIGH(temp);
			PWM = (PWM >> 4); // PWM divided by 16
			result = (LOW(PWM)+ LocoData.CV2);
//				PWM = (LocoData.Speed*(LocoData.CV5-LocoData.CV2)/28) + LocoData.CV2;
		}
	} else { // 128 Speed Step
		if (LocoData.Speed>1) { // Speed below or equal to 3 is Stop or EStop
			if (LocoData.CV6>1){ // Must apply the VMid parameter for PWM out
				if (LocoData.Speed<0x40) {
					 // PWM = Spd * ( VMid - VMin) / 0x3F  + VMin
					PWM = (LocoData.Speed-1)*(LocoData.CV6-LocoData.CV2) >> 6;
//					PWM = PWM >> 6;	
					result = PWM + LocoData.CV2;
					
				}	else {
					 // PWM = Spd * ( VMax - VMid) / 0x3F  + VMin
					PWM = (LocoData.Speed-0x40)*(LocoData.CV5-LocoData.CV6) >> 6;			
//					PWM = PWM >> 6;
					result = PWM + LocoData.CV6;
				}
			} else { // There is no VMid configured so proceede with the normal linear calculation
				PWM = (LocoData.Speed-1)*(LocoData.CV5-LocoData.CV2) << 1;			
				result = HIGH(PWM)+ LocoData.CV2;
			}
		}
	}
	return (result);
}


/******************************************************************************
 * Function:        unsigned char CheckEEPromAndRam(void)
 * PreCondition:    None
 * Input:           None
 * Output:          Char : 00 : There is no Difference
 *												 01 : There is at least one difference
 * Side Effects:    None
 * Overview:        Checks the LocoData EEProm and RAM values to see if there is
 *									any difference. It is intended to detect some unwanted EEPROM values
 *
 * Note:            
 *****************************************************************************/
unsigned char CheckEEPromAndRam(void) {
  unsigned char eeprom_address=INIT_EEPROM_ADDR;
  unsigned char * RamData;
  unsigned char i;
  unsigned char result;
  
  result = 0x00; // Assumes there are no differences
  RamData = &LocoData.Address;
  for (i=0;i<MAX_CV_VARIABLES;i++) {
    if (*RamData != EEPROM_READ(eeprom_address)) {
			result = 0xFF; // Found the difference
			break;
	  } 
    RamData++;
    eeprom_address++;
  }
  
	return result;
}
	
/******************************************************************************
 * Function:        unsigned char DecodeSpeed(char CodedSpeed)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        Decode the received Speed from DCC, depending 
 *									wether it is 28 or 128.
 *
 * Note:            
 *****************************************************************************/
unsigned char DecodeSpeed(char CodedSpeed) {
	unsigned char result;

	LocoData.Flags.bits.DirBit = 0; //Assumes forward direction
	if (LocoData.Flags.bits.SpeedMode == _28SPEED) {
		if (BTSTF(CodedSpeed,5)==1)
			LocoData.Flags.bits.DirBit =1; // Change to reverse direction
		result = (CodedSpeed & 0b00001111) << 1;
		if (BTSTF(CodedSpeed,4)) {
			result++;  // Sets the LSb of the Speed
		}
	} else {
		if (BTSTF(buffer[2],7)==1) {
			LocoData.Flags.bits.DirBit=1; // Change to reverse direction
		}
		result = CodedSpeed & 0x7F;// Masks the 7 bit speed
	}

	if (LocoData.CV29.flags.LocoDirection == 1) {// Reversing mode
	  LocoData.Flags.bits.DirBit ^= 1; // Toggle Direction Bit
	}

	DCCFlags.flag.RecalcPWM = 1; // Flag to calculate the PWM time ON in Main routine
	return (result);

}	



/******************************************************************************
 * Function:        void InitDCCState(void);
 * PreCondition:    None
 * Input:           
 * Output:          
 * Side Effects:    None
 * Overview:        Initialize the DCC Variables
 * Note:            None
 *****************************************************************************/
void InitDCCState(void) {
// ***** Initiates DCC States *****	
	DCCFlags._byte = 0;
	DCCFlags.flag.DCMode = 0; // Assumes is operating on DCC. Changes when received the first DCC bit
	LocoData.Flags.bits.SpeedMode = _28SPEED;
	PreambleBits =0;
	DCC_Byte=0;
	DCC_State=0;
	BytesReceived=0;
}


/******************************************************************************
 * Function:        void LoadEEpromData(void);
 * PreCondition:    None
 * Input:           
 * Output:          
 * Side Effects:    None
 * Overview:        Load CV from EEPROM
 * Note:            None
 *****************************************************************************/
void LoadEEpromData(void) {
  char eeprom_address=INIT_EEPROM_ADDR;
  char * RamData;
  char i;
  
  RamData = &LocoData.Address;
  for (i=0;i<MAX_CV_VARIABLES;i++) {
    *RamData=EEPROM_READ(eeprom_address);
    RamData++;
    eeprom_address++;
  }
} 



/******************************************************************************
 * Function:        void CheckDCTimeOut(void);
 * PreCondition:    None
 * Input:           
 * Output:          
 * Side Effects:    None
 * Overview:        Checks for 30 ms overflow of DCTimeOut register
 *									to detect an stop or absense of the DCC signal for longer
 *									periods. The decoder always start wating for DCC signal but
 *									if not received a valid bit in more than 20ms (according to NMRA standart)
 *									then it will switch to DC mode if enabled on CV29 bit 2 (Power Convertion)
 *									This state will go back to DCC if a valid DCC bit are received 
 *									any given time.
 * Note:            None
 *****************************************************************************/
void CheckDCTimeOut(void) {
		DCTimeOut++;
		if (DCTimeOut>30) {
			DCTimeOut=0;
			if (LocoData.CV29.flags.PowerConvertion == 1) 
				DCCFlags.flag.DCMode = 1; // DC Mode Enabled
			else
				DCCFlags.flag.DCMode = 0; // DC Mode Disabled
			
			DCCFlags.flag.DCC_bit_state=0; // Reset the Bit status for receiving DCC signals
			DCCFlags.flag.CheckFullBit=0; 
		}
		
}

/******************************************************************************
 * Function:        void main(void)
 * PreCondition:    None
 * Input:           
 * Output:          
 * Side Effects:    None
 * Overview:        
 * Note:            None
 *****************************************************************************/
void main(void){
	unsigned int i;
	char j;
// **** Initiates the hardware ****		
	initPorts();
	CMCON = 0x07; // Disable the compare module and enable portA as digital
	PWM_Timer = 0;
	//EECON1=0; // Clears WREN to prevent undesired EEPROM writes (Hope this solves the problem!!!)

	if(PCONbits.nPOR == 0) { // Power On Reset Has ocurred

			PWM_Port1 = 0; 
			PWM_Port2 = 0;
			PWM_RealTOn = 0;
			PWM_TOn = 0;
			PWM_Timer = 0;
			LocoData.Speed = 0;

	} else {
		if (PCONbits.nBOR == 0) { // Brown out reset action
			PWM_Port1 = 0;
			PWM_Port1 = 0;
			LED_REAR = 1;
			LED_FRONT = 1;
			LocoData.Speed = 0;
		}
		if (STATUSbits.nTO == 0) {
			// Watch dog Timer reset action
			PWM_Port1 = 0;
			PWM_Port1 = 0;
			LED_REAR = 1;
			LED_FRONT = 1;
			LocoData.Speed = 0;
		}	
	}
	STATUSbits.nTO =1; // WDT Flag
	PCONbits.nBOR  =1; // Brown Out Detect Flag
	PCONbits.nPOR  =1; // Power On Reset
	STATUSbits.nPD =1;

	OPTION_REG	= 0b00000001;  // Preescaler x 4 assigned to TMR0
	TMR0 = PWM_CLICK_TIMER;
	PEIE = 1; // To enable Periferial Interrupts (CCP1)
	T0IE = 1; // Timer 0 Interrupt Enabled
	TMR1IE = 1; // Enable the Timer 1 interrupt to detect long idle bits and restart DCC bit sequence
	TMR2IE = 1; // Enable the Timer 2 interrupt for acc & decceleration timming
	CCP1IE = 1; // Capture Interrupt Enabled - for dcc signal timming
	if (DCC_Input == 1)
	  CCP1CON = 0b00000100; // Capture on Falling Edge (for now
	else
	  CCP1CON = 0b00000101; // Capture on Rising Edge (for now
	
	T1CON	= 0b00100101; // Preescaler x 2
	TMR1 = 0; // Clears the Timer 1 (used to capture the DCC timming sequence)
	PR2	 = ACCEL_DECEL_BASE; // 1 ms timer 
	T2CON = 0b01111111;
	TMR2 = 0;

	PWM_Timer = 0;
	PWM_Period = PWM_MAX;
	
	InitDCCState();
	ACKTime = 0; // Initiates the AckTime to zero so It wont turn the motor
	LoadEEpromData();

	GIE= 1;		// Enable Global Interrupts
		
	while (1) {
		if (DCCFlags.flag.DCMode == 1) {
			PWM_TOn = PWM_MAX; // Max PWM Speed but the motor speed depends on the DC voltage applied to the track
			if (DCC_Input == 1) { // Check the Track Polarity
				LocoData.Flags.bits.DirBit=1;
			} else {
				LocoData.Flags.bits.DirBit=0;
			}
		}
		
		// Time consuming routines that only executes when a DCC packet was analized
		if ((DCCFlags.flag.MustProcessCmd == 1) && !DCCFlags.flag.CheckHalfBit) {
			DCCFlags.flag.MustProcessCmd = 0;
			PWM_TOn = CalculatePWMOut(); // Recalculates the PWM TOn to prevent runaways
		}
		
		if (DCCFlags.flag.CheckLights == 1 && !DCCFlags.flag.CheckHalfBit) { // Executes every 1ms
			CLRWDT();
		  DCCFlags.flag.CheckLights =0;
		  CheckLights();
		  CalcAccelerate(); // Also recalculates the acceleration - deceleration
			CheckDCTimeOut();
			
			// Checks for ServiceMode Time Out (20 ms According to NMRA 9.2.3)
			ServModeTimer++;
			if (ServModeTimer > 20) {
				ServModeTimer=0;
				DCCFlags.flag.DCCServiceMode=0;// Exits ServiceMode
				DCCFlags.flag.MayProcServMode=0;	
				ServModeCount=0;
			}

			// Checks if a programming packet has succesfully hit the decoder and Acknowled the reception
			if (DCCFlags.flag.Ack == 1) {
				ACKTime++; 
				if (ACKTime < MAX_ACK_TIME) {
					LocoData.Speed = 0;
					LocoData.Flags.bits.RealDirBit=1;
					LocoData.Flags.bits.DirBit=1;
					if (LocoData.Flags.bits.RealDirBit==1){
			  		PWM_Port1 = 1;	
						PWM_Port2 = 0;
			  	} else {
						PWM_Port1 = 0;
			  		PWM_Port2 = 1;	
			  	}
				}	else {
					ACKTime = 0;
					PWM_Port1 =0;
					PWM_Port1 =0;
					DCCFlags.flag.Ack = 0;
					DCCFlags.flag.RecalcPWM = 1;
					LocoData.Speed = 0;
				}
			}	
		}
		
		if (DCCFlags.flag.CheckHalfBit == 1) {
			DCCFlags.flag.CheckHalfBit = 0; // Clears flag
			if (DCCFlags.flag.DCC_bit_state == 0) { // First half bit state
			  if ((HalfBitTime > _1LOWER_LIMIT)&&(HalfBitTime< _1UPPER_LIMIT))
			    DCCFlags.flag.FirstHalfBit = 1;
			  else
			  	DCCFlags.flag.FirstHalfBit = 0;
			} else {
			  if ((HalfBitTime > _1LOWER_LIMIT)&&(HalfBitTime< _1UPPER_LIMIT))
			    DCCFlags.flag.BitReceived = 1;
			  else
			  	DCCFlags.flag.BitReceived = 0;
			  
			  if (DCCFlags.flag.BitReceived == DCCFlags.flag.FirstHalfBit) {
					  DCCFlags.flag.CheckFullBit = 1;  
				} else {  // the two half bits detected were different
					if (DCCFlags.flag.FirstHalfBit == 0) {
						DCCFlags.flag.DCC_bit_state = 1;
					} else
						DCCFlags.flag.DCC_bit_state = 0;
					DCCFlags.flag.FirstHalfBit = DCCFlags.flag.BitReceived;
					//DCCFlags.flag.DCC_bit_state ^= 1; // Syncronize with the first half bit
					DCC_State = 0 ; // Resets the DCC state machine
					PreambleBits = 0; //Resets the preamble bits because a difference in timming of the 2 half bits
				}  
			}
			DCCFlags.flag.DCC_bit_state ^= 1; // Toggle the bit_state
			if (DCCFlags.flag.CheckFullBit) {
				if (DCCFlags.flag.DCMode == 1) { // If was in DC Mode then init the DCC State
					DCCFlags.flag.DCMode = 0; // Clear the DC flag to run on DCC
					InitDCCState();
					PWM_Timer = 0;
					PWM_Period = PWM_MAX;
					PWM_TOn = 0;
				}
				DCCFlags.flag.CheckFullBit=0; // Clears the flag
			  switch (DCC_State) {
				  case PREAMBLE_STATE : {
					  if (DCCFlags.flag.BitReceived == 1) {
					    PreambleBits++;
						} else {
							PreambleBits=0; // Resets the Preamble Counter	
				 			DCTimeOut=0; // Clears the DC Time out of 20ms(Timer2)
						}
					  if (PreambleBits >= 10) {
					    DCC_State++; // The minimum preamble bits has been received
					  }  
					  break;
					} // end of case PREAMBLE_STATE 
				  
				  case WAIT_INIT_BIT : {
					  if (DCCFlags.flag.BitReceived == 1) {
							PreambleBits++; // Continue incrementing Preamble bits for Long preamble packets
						} else {
							DCC_State++; // A Start (0) bit has been received
							BytesReceived=0;
							buffer[0]=0;
							buffer[1]=0;
							buffer[2]=0;
							buffer[3]=0;
							buffer[4]=0;
						}
						break;
					}
					
					case FIRST_BYTE_STATE : {
						buffer[0] |= DCCFlags.flag.BitReceived; 
						BitsReceived++;
						if (BitsReceived < 8) {
							buffer[0] = buffer[0] << 1;
						} else {
							DCC_State++; // Checks for the 0 bit between bytes
							BitsReceived =0;
							BytesReceived++;

						}
						break;	
					}
					
					case WAIT_START1_BIT : {
						if (DCCFlags.flag.BitReceived == 0) {
							DCC_State++;
						} else {
							DCC_State =0; // An error on the protocol ocurred so restarts the packet	
							PreambleBits=0;
						}
						break;	
					}
					
					case SECOND_BYTE_STATE : {
						buffer[1] |= DCCFlags.flag.BitReceived; 
						BitsReceived++;
						if (BitsReceived < 8) {
							buffer[1] = buffer[1] << 1;
						} else {
							DCC_State++; // Checks for the 0 bit between bytes
							BitsReceived =0;	
							BytesReceived++;
						}
						break;	
					}

					case WAIT_START2_BIT : {
						if (DCCFlags.flag.BitReceived == 0) {
							DCC_State++;
						} else {
							DCC_State =0; // An error on the protocol ocurred so restarts the packet	
							PreambleBits=0;
						}
						break;	
					}

					case THIRD_BYTE_STATE : {
						buffer[2] |= DCCFlags.flag.BitReceived; 
						BitsReceived++;
						if (BitsReceived < 8) {
							buffer[2] = buffer[2] << 1;
						} else {
							DCC_State++; // Checks for the 0 bit between bytes
							BitsReceived =0;	
							BytesReceived++;
						}
						break;	
					}
					
					case WAIT_START3_BIT : {
						if (DCCFlags.flag.BitReceived == 0) {
							DCC_State++;
						} else {
							AnalizeDCC();
							DCC_State = 0; // An error on the protocol ocurred so restarts the packet	
							PreambleBits=0;
						}
						break;	
					}
					
					case FORTH_BYTE_STATE : {
						buffer[3] |= DCCFlags.flag.BitReceived; 
						BitsReceived++;
						if (BitsReceived < 8) {
							buffer[3] = buffer[3] << 1;
						} else {
							DCC_State++; // Checks for the 0 bit between bytes
							BitsReceived =0;	
							BytesReceived++;
						}
						break;	
					}
					
					case WAIT_START4_BIT : {
						if (DCCFlags.flag.BitReceived == 0) {
							DCC_State++;
						} else {
							AnalizeDCC();
							DCC_State = 0; // An error on the protocol ocurred so restarts the packet	
							PreambleBits=0;
						}
						break;	
					}

					case FIFTH_BYTE_STATE : {
						buffer[4] |= DCCFlags.flag.BitReceived; 
						BitsReceived++;
						if (BitsReceived < 8) {
							buffer[4] = buffer[4] << 1;
						} else {
							DCC_State++; // Checks for the 0 bit between bytes
							BitsReceived =0;	
							BytesReceived++;
						}
						break;	
					}
					
					case WAIT_START5_BIT : {
						if (DCCFlags.flag.BitReceived == 1) { // End Packet Bit (
							AnalizeDCC();
							DCC_State = 0;
						} else {
							DCC_State =0; // Resets the Packet State because It didnt received and 1 end bit
							PreambleBits=0;
						}
						break;	
					}
					
					default : DCC_State = 0;

				} // end of  switch (DCC_State)
			} // End if(LocoFlags.flags.CheckFullBit)
		}

	}
		
}	
